﻿
using System;
using System.Drawing;
using System.Windows.Forms;

/*                    Hermitian interpolation               */
/*                        www.mosismath.com                 */


namespace WindowsFormsApplication1
{

   
    public partial class MainWin : Form
    {
        public const int order =  6; // number of reference points
        public const int datapoints = 1001; // number of datapoints for the interpolation
        public const double deltaX = 1.5; // sample time 

        double[] y_k = new double[order];  // supporting points
        double[] x_k = new double[order];
        double[] p_k = new double[order];
        double[] yp = new double[datapoints]; // data points for the interpolation in the graph
        double[] xp = new double[datapoints];


        public MainWin()
        {
            int i;
            DataGridViewCell cell;

            InitializeComponent();

            dGPolynom.ColumnCount = order;
            dGPolynom.RowCount = 3;
            cell = dGPolynom.Rows[0].HeaderCell;
            cell.Value = "y_k";
            cell = dGPolynom.Rows[1].HeaderCell;
            cell.Value = "x_k";
            cell = dGPolynom.Rows[1].HeaderCell;
            cell.Value = "p_k";

            // init some sample y values
            y_k[0] = 0.5;
            y_k[1] = 3.7;
            y_k[2] = 3.8;
            y_k[3] = 4.0;
            y_k[4] = 2.2;
            y_k[5] = 0.2;

            p_k[0] = 2.5;
            p_k[1] = 1.5;
            p_k[2] = -1.1;
            p_k[3] = -0.2;
            p_k[4] = -3.0;
            p_k[5] = -1.0;

            // timestamp to y value
            for (i = 1; i <= order; i++)
            {
                x_k[i - 1] = deltaX * (double)i;
            }

            for (i = 0; i < order; i++)
            {
                dGPolynom.Columns[i].Name = "P" + i.ToString();
                cell = dGPolynom[i, 0];
                cell.Value = Convert.ToString(y_k[i]);
                cell = dGPolynom[i, 1];
                cell.Value = Convert.ToString(x_k[i]);
                cell = dGPolynom[i, 2];
                cell.Value = Convert.ToString(p_k[i]);
            }
        }


        double Hermite(double y1, double y2, double p1, double p2, double x)
        {
            return y1 * (1 + 2 * x) * (1 - x) * (1 - x) + y2 * (3 - 2 * x) * x * x + p1 * (x - 1) * (x - 1) * x + p2 * (x - 1) * x * x;
        }


        public void DrawGraph(int maxPoints, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if (maxTime < xp[j])
                    maxTime = xp[j];
                if (maxVal < yp[j])
                    maxVal = yp[j];
                if (minVal > yp[j])
                    minVal = yp[j];
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            if (scalefactor > pGraph.Width / (double)maxTime)
                scalefactor = pGraph.Width / (double)maxTime;
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    p1.X = Convert.ToInt32(xp[j] * scalefactor);
                    p2.X = Convert.ToInt32(xp[j + 1] * scalefactor);
                    if (drawCeroline)
                    {
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                        p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                    }
                    else
                    {
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                        p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                    }
                    g.DrawLine(bluePen, p1, p2);
                }
                for (j = 0; j < order; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * scalefactor);
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    else
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();
        }

        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i, j;
            DataGridViewCell cell;
            int actPoints = (datapoints-1) / (order-1);

            for (i = 0; i < order; i++)
            {
                cell = dGPolynom[i, 0];
                if (cell.ToString() != "")
                {
                    try
                    {
                        y_k[i] = Convert.ToDouble(cell.Value);
                        cell = dGPolynom[i, 1];
                        x_k[i] = Convert.ToDouble(cell.Value);
                        cell = dGPolynom[i, 2];
                        p_k[i] = Convert.ToDouble(cell.Value);
                    }
                    catch
                    {
                        y_k[i] = 0.0;
                        x_k[i] = 0.0;
                        p_k[i] = 0.0;
                    }
                }
                else
                {
                    y_k[i] = 0.0;
                    x_k[i] = 0.0;
                    p_k[i] = 0.0;
                }
            }
            for (i = 0; i < order-1; i++)
            {
                for (j = 0; j <= actPoints; j++)
                {
                    xp[j + i * actPoints] = x_k[i] + (x_k[i + 1] - x_k[i])*(double)(j) / actPoints;
                    yp[j + i * actPoints] = Hermite(y_k[i], y_k[i + 1], p_k[i], p_k[i + 1], (double)j / actPoints);     
                }
            }
            DrawGraph(datapoints, true);
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, false);
        }  
    }
}
